STYLESHEETS.append(data("cubes.conference.css"))
NOATTEND_ICON = data("noattend.jpeg")
ATTEND_ICON = data("attend.jpeg")

# Talk ressources
workflowColorAction = "#e5f5ee"
workflowColorWait = "#f5e5ee"
